#include "GraphicsGL.h"
#pragma hdrstop

namespace Spr {;

SGOBJECTIMP(GLRender, GRRender);

GLRender::GLRender(){
	nLights = 0;
}
GLRender::~GLRender(){
}
///	_O(_܂)
void GLRender::Render(SGScene* s){
	//	_s̐ݒ
    if (camera){
		glMatrixMode(GL_MODELVIEW);
		glLoadMatrixf(camera->data.view);
	}
	//	s̕`
	drawState = DRAW_OPAQUE;
	glBlendFunc(GL_ONE, GL_ZERO);
	RenderRecurse(s->GetWorld());
	//	Engine̕`
	s->GetBehaviors().Render(this, s);
	//	̕`
	drawState = DRAW_TRANSPARENT;
	glDepthMask(GL_FALSE);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	RenderRecurse(s->GetWorld());
	glDepthMask(GL_TRUE);
	drawState = DRAW_BOTH;
	glBlendFunc(GL_ONE, GL_ZERO);
}
///	_O(ċA)
void GLRender::RenderRecurse(SGFrame* n){
	//	Worlds̐ݒ
	glPushMatrix();
	glMultMatrixf(n->GetPosture());
	//	VisualƎqt[̕`
	GRRender::RenderRecurse(n);
	//	Worlds߂
	glPopMatrix();
}
///	obt@NA
void GLRender::ClearBuffer(){
	glClearColor(0,0,0,1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}
///	_O̊JnOɌĂԊ֐
void GLRender::BeginScene(){
}
///	_ȌIɌĂԊ֐
void GLRender::EndScene(){
	glFlush();
}
void GLRender::Resize(Vec2f screen){
	glViewport(0,0, (size_t)screen.X(), (size_t)screen.Y());
	Affinef afProj;
	afProj = Affinef::ProjectionGL(Vec3f(camera->data.center.X(), camera->data.center.Y(), camera->data.front),
		Vec2f(camera->data.size.X(), camera->data.size.Y() * screen.Y()/screen.X() ), camera->data.front, camera->data.back);
	SetProjectionMatrix(afProj);
}

void GLRender::SetViewMatrix(const Affinef& afv){
	if (camera) camera->data.view = afv;
	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(camera->data.view * afModel);
}
void GLRender::SetModelMatrix(const Affinef& afw){
	afModel = afw;
	glMatrixMode(GL_MODELVIEW);
	if (camera){
		glLoadMatrixf(camera->data.view * afModel);
	}else{
		glLoadMatrixf(afModel);
	}
}
void GLRender::SetProjectionMatrix(const Affinef& afp){
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(afp);
	glMatrixMode(GL_MODELVIEW);
}
void GLRender::DrawDirect(TPrimitiveType ty, Vec3f* begin, Vec3f* end){
	GLenum mode;
	if (ty == POINTS) mode = GL_POINTS;
	else if (ty == LINES) mode = GL_LINES;
	else if (ty == TRIANGLES) mode = GL_TRIANGLES;
	else assert(0);
	glBegin(mode);
	for(;begin!=end; ++begin) glVertex3fv(*begin);
	glEnd();
}
void GLRender::DrawIndexed(TPrimitiveType ty, size_t* begin, size_t* end, Vec3f* vtx){
	GLenum mode;
	if (ty == POINTS) mode = GL_POINTS;
	else if (ty == LINES) mode = GL_LINES;
	else if (ty == TRIANGLES) mode = GL_TRIANGLES;
	else assert(0);
	glBegin(mode);
	for(;begin!=end; ++begin) glVertex3fv(vtx[*begin]);
	glEnd();
}
void GLRender::SetMaterial(const GRMaterialData& m){
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, m.ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, m.diffuse);
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, m.specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, m.emissive);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, m.power);
}
void GLRender::PushLight(const GRLightData& l){
	if (nLights < GL_MAX_LIGHTS){
		glLightfv(GL_LIGHT0+nLights, GL_AMBIENT, l.ambient);
		glLightfv(GL_LIGHT0+nLights, GL_DIFFUSE, l.diffuse);
		glLightfv(GL_LIGHT0+nLights, GL_SPECULAR, l.specular);
		glLightfv(GL_LIGHT0+nLights, GL_POSITION, l.position);
		if (l.position.W()){
			glLightf(GL_LIGHT0+nLights, GL_CONSTANT_ATTENUATION, l.attenuation0);
			glLightf(GL_LIGHT0+nLights, GL_LINEAR_ATTENUATION, l.attenuation1);
			glLightf(GL_LIGHT0+nLights, GL_QUADRATIC_ATTENUATION, l.attenuation2);
			glLightfv(GL_LIGHT0+nLights, GL_SPOT_DIRECTION, l.spotDirection);
			glLightf(GL_LIGHT0+nLights, GL_SPOT_EXPONENT, l.spotFalloff);
			glLightf(GL_LIGHT0+nLights, GL_SPOT_CUTOFF, l.spotCutoff);
		}
		glEnable(GL_LIGHT0+nLights);
	}
	nLights ++;
}
void GLRender::PopLight(){
	nLights --;
	if (nLights < GL_MAX_LIGHTS) glDisable(GL_LIGHT0+nLights);
}
void GLRender::SetLineWidth(float w){
	glLineWidth(w);
}
void GLRender::SetDepthTest(bool b){
	if (b) glEnable(GL_DEPTH_TEST);
	else glDisable(GL_DEPTH_TEST);
}
void GLRender::SetDepthFunc(TDepthFunc f){
	DWORD glf;
	switch(f){
	case DF_NEVER:		glf = GL_NEVER; break;
	case DF_LESS:		glf = GL_LESS; break;
	case DF_EQUAL:		glf = GL_EQUAL; break;
	case DF_LEQUAL:		glf = GL_LEQUAL; break;
	case DF_GREATER:	glf = GL_GREATER; break;
	case DF_NOTEQUAL:	glf = GL_NOTEQUAL; break;
	case DF_GEQUAL:		glf = GL_GEQUAL; break;
	case DF_ALWAYS:		glf = GL_ALWAYS; break;
	}
	glDepthFunc(glf);
}
bool GLRender::CanDraw(){
	return true;
}
void GLRender::Loaded(SGScene* scene){
	InitTree(scene->GetWorld(), scene);
}
void GLRender::InitTree(SGFrame* fr, SGScene* scene){
	for(unsigned i=0; i<fr->contents.size(); ++i){
		GRMesh* gmesh = DCAST(GRMesh, fr->contents[i]);
		if (gmesh){
			UTRef<GLMesh> mesh = new GLMesh;
			fr->contents.Push(mesh);
			mesh->Set(gmesh, scene);
			continue;
		}
		GRSphere* gsphere = DCAST(GRSphere, fr->contents[i]);
		if (gsphere){
			UTRef<GLSphere> sphere = new GLSphere;
			fr->contents.Push(sphere);
			sphere->Set(gsphere, scene);
			continue;
		}
	}
	for(unsigned i=0; i<fr->Children().size(); ++i){
		InitTree(fr->Children()[i], scene);
	}
}

}	//	Spr


