#include "Physics.h"
#pragma hdrstop

using namespace PTM;
namespace Spr{;

void LimitCycle(float& t){
	if (!_finite(t) || t > 1e6 || t < -1e6){
		if (t>0) t = (float)M_PI;
		else t= - (float)M_PI;
		return;
	}
	int times;
	if (t > M_PI){
		times = int((t+M_PI) / (2*M_PI));
	}else if (t < -M_PI){
		times = int((t-M_PI) / (2*M_PI));
	}else{
		return;
	}
	t -= (float)(times*(2*M_PI));
	assert(-M_PI < t && t < M_PI);
}

////////////////////////////////////////////////////////////////////////////////
//c[
PHJoint* PHJoint::Search(PHSolid* s)
{
	if(solid == s)return this;
	PHJoint* pNode;
	array_type::const_iterator it;
	for(it = Children().begin(); it != Children().end(); it++){
		pNode = (*it)->Search(s);
		if(pNode)return pNode;
	}
	return 0;
}

int PHJoint::NumNodes()
{
	//fixed link̃[gm[h͐Ȃ
	int nNodes = solid ? 1 : 0;
	for(array_type::const_iterator it = Children().begin(); it != Children().end(); it++)
		nNodes += (*it)->NumNodes();
	return nNodes;
}
int PHJoint::NumJoints()
{
	int nNodes = GetParent() ? 1 : 0;
	for(array_type::const_iterator it = Children().begin(); it != Children().end(); it++)
		nNodes += (*it)->NumJoints();
	return nNodes;
}

//////////////////////////////////////////////////////////
//	
SGOBJECTIMP(PHJoint, SGObject);

PHJoint::PHJoint():solid(0), frame(0){}
PHJoint::PHJoint(PHSolid* s):solid(s), frame(0){}
PHJoint::PHJoint(SGFrame* f):solid(0), frame(f){}
PHJoint::PHJoint(PHSolid* s, const PHJointInfo& i):solid(s), frame(0), info(i){}

void PHJoint::Loaded(SGScene* scene)
{
	//solidframew̏ꍇ̏
	if(!solid && !frame){
		solid = new PHSolid;
		solid->SetMass(1e-3f);
		solid->SetInertia(1e-3f * Matrix3d());
		solid->SetFrame(new SGFrame);
	}
	if(solid)solid->Loaded(scene);
	
	//ϐ̏
	SetInfo(INFO_SOLID);
	if(GetParent()){
		CompJointAxis();
		//덷ƌ덷ݐς[h/Z[ȗΏۂɂׂȂ񂾂ˁEE
		error = error_int = 0.0;
	}
	else{
		if(solid){
			p = solid->GetCenterPosition();
			R = solid->GetRotation();
			v_abs = solid->GetVelocity();
			w_abs = solid->GetAngularVelocity();
		}else{
			p = frame->GetPosition();
			R = frame->GetRotation();
			v_abs.clear();
			w_abs.clear();
		}		
	}
	Ia.clear();
	Za.clear();

	Reset();

	for(unsigned i=0; i<Children().size(); ++i){
		Children()[i]->Loaded(scene);
	}
}

//{Iȏԗ(PHJointState̓e)LoadedLoadStateŃZbgꂽɁA
//]ϐvZ邽߂̊֐
void PHJoint::Reset(){
	//񃋁[gm[h
	if(GetParent()){	
		PropagateState();
		solid->SetCenterPosition(p);
		solid->SetRotation(R);
	}
	//[gm[h
	else{
		quat.from_matrix(R);
		v = R.trans() * v_abs;
		w = R.trans() * w_abs;
	}
}

//////////////////////////////////////////////////////////
//E؍\
SGObject* PHJoint::ChildObject(size_t i){
	return Children()[i];
}
SGObject* PHJoint::ReferenceObject(size_t i){
	if (i!=0) return NULL;
	if (solid) return solid;
	return frame;
}
size_t PHJoint::NReferenceObjects(){
	if (frame || solid) return 1; 
	return 0;
}
size_t PHJoint::NChildObjects(){
	return Children().size();
}

bool PHJoint::AddChildObject(SGObject* o, SGScene* s){
	if (DCAST(PHSolid, o)){
		solid = (PHSolid*)o;
		return true;
	}
	if (DCAST(PHJoint, o)){
		((PHJoint*)o)->SetParent(this);
		return true;
	}
	return false;
}

bool PHJoint::DelChildObject(SGObject* o, SGScene* s){
	PHJoint* j = DCAST(PHJoint, o);
	for(unsigned i=0; i<Children().size(); ++i){
		if (Children()[i] == j){
			j->SetParent(NULL);
			return true;
		}
	}
	return false;
}

const UTTypeInfo** PHJoint::ChildCandidates(){
	static const UTTypeInfo* rv[] = {
		PHJoint::GetTypeInfoStatic(), NULL
	};
	return rv;
}

//////////////////////////////////////////////////////////
//E̍XV
void PHJoint::SetInfo(PHJointInfoEntry entry, bool recursive, void* value)
{
	//solid̏ǂݍ
	if(entry == INFO_SOLID){
		if(solid){
			m = solid->GetMass();
			I = solid->GetInertia();
		}
		else{
			m = 0.0;
			I.clear();
		}
		Ii.clear();
		smitem(Ii, 0, 1) = Matrix3d::Diag(m, m, m);
		smitem(Ii, 1, 0) = I;
	}

	if(!GetParent() || !value)return;

	switch(entry){
	case INFO_POSITIONPARENT: info.v3fPositionParent = *(Vec3f*)value; break;
	case INFO_ROTATIONPARENT: info.m3fRotationParent = *(Matrix3f*)value; break;
	case INFO_POSITIONCHILD: info.v3fPositionChild = *(Vec3f*)value; break;
	case INFO_ROTATIONCHILD: info.m3fRotationChild = *(Matrix3f*)value; break;
	case INFO_MAXTORQUE: info.fMaxTorque = max(0.0f, *(float*)value); break;
	case INFO_MINPOSITION: info.fMinPosition = *(float*)value; break;
	case INFO_MAXPOSITION: info.fMaxPosition = *(float*)value; break;
	case INFO_INPUTTYPE: info.nInputType = *(DWORD*)value; break;
	case INFO_INPUT: info.fInput = *(float*)value; break;
	case INFO_PVALUE: info.fPValue = max(0.0f, *(float*)value); break;
	case INFO_IVALUE: info.fIValue = max(0.0f, *(float*)value); break;
	case INFO_DVALUE: info.fDValue = max(0.0f, *(float*)value); break;
	default:;
	}

	//֐߈ʒuύXꍇ
	switch(entry){
	case INFO_POSITIONPARENT:
	case INFO_ROTATIONPARENT:
	case INFO_POSITIONCHILD:
	case INFO_ROTATIONCHILD:
		CompJointAxis();
		CompRelativePosition();
		//TODO:ŉʃm[ḧʒuׂčXV
	}

	//ċAw̏ꍇ
	if(recursive)
		for(array_type::const_iterator it = Children().begin(); it != Children().end(); it++)
			(*it)->SetInfo(entry, recursive, value);
}

/////////////////////////////////////////////////////////////////////
//

void PHJoint::SetPosition(double pos)
{
	info.nInputType = TYPE_POSITION;
	info.fInput = (float)pos;
	//]֐߂̏ꍇ͓͒l[-,]͈̔͂ɐ
	if(info.nType == TYPE_HINGE){
		LimitCycle(info.fInput);
	}
}
void PHJoint::SetVelocity(double vel)
{
	info.nInputType = TYPE_VELOCITY;
	info.fInput = (float)vel;
}
void PHJoint::SetAccel(double accel)
{
	info.nInputType = TYPE_ACCEL;
	info.fInput = (float)accel;
}
void PHJoint::SetTorque(double torque)
{
	info.nInputType = TYPE_TORQUE;
	info.fInput = (float)torque;
}

void PHJoint::CompTorque(double dt)
{
	float e;
	switch(info.nInputType){
	case TYPE_TORQUE:
		torque = info.fInput;
		break;
	case TYPE_ACCEL:
		//̉xo߂̃gNvZ
		torque = info.fInput * dot_s_Ia_s + dot_s_Z_plus_Ia_c + svdot(s, Ia * a_p);
		break;
	case TYPE_VELOCITY:
		//xPID
		e = info.fInput - info.fVelocity;
		error_diff = (e - error) / dt;
		error_int += e;
		error = e;
		torque = info.fPValue * error + info.fIValue * error_int + info.fDValue * error_diff;
		break;
	case TYPE_POSITION:
		//ʒuPID
		e = info.fInput - info.fPosition;
		if(info.nType == TYPE_HINGE){
			LimitCycle(e);
		}
		error_diff = (e - error) / dt;
		error_int += e;
		error = e;
		torque = info.fPValue * error + info.fIValue * error_int + info.fDValue * error_diff;
		break;
	default:
		torque = 0.0;
	}

	//őgNŃNv
	if(info.fMaxTorque >= 0.0){
		if(torque >  info.fMaxTorque)torque =  info.fMaxTorque;
		if(torque < -info.fMaxTorque)torque = -info.fMaxTorque;
	}
}

/////////////////////////////////////////////////////////////////////
//Featherstone's algorithm

void PHJoint::Dynamics(double dt)
{
	CompArticulatedInertia(dt);
	Integrate(dt);
}

////////////////////////////////////////////////////////
//EArticulated InertiavZ
//EArticulated 0-accel forcevZ
void PHJoint::CompArticulatedInertia(double dt)
{
	//qm[hɌvZ
	for(array_type::const_iterator it = Children().end(); it != Children().begin();)
		(*--it)->CompArticulatedInertia(dt);

	//悭goĂ鎮LbV
	rcross = Matrix3d::Cross(prc);
	rpcross = Matrix3d::Cross(pRc * prc);
	rcross_cRp = rcross * cRp;
	rpcross_pRc = rpcross * pRc;
		
	//Za
	if(solid){
		svitem(Za, 0) += -R.trans() * solid->GetForce();
		svitem(Za, 1) += -R.trans() * solid->GetTorque() + Vec3d(
			(I[2][2] - I[1][1]) * w.Y() * w.Z(),
			(I[0][0] - I[2][2]) * w.Z() * w.X(),
			(I[1][1] - I[0][0]) * w.X() * w.Y());
	}
	//Ia
	Ia += Ii;

	if(GetParent()){
		//RIx
		CompCoriolisAccel();
		//gN
		CompTorque(dt);
	}

	//LbV
	smat = svmat(s, s);
	Ia_c = Ia * c;
	Ia_s = Ia * s;
	Z_plus_Ia_c = Za + Ia_c;
	dot_s_Ia_s = svdot(s, Ia_s);
	dot_s_Z_plus_Ia_c = svdot(s, Z_plus_Ia_c);
		
	//em[hZa,IaɎZa,Iaςݏグ
	if(GetParent()){
		//Ia
		GetParent()->Ia += pXc_Mat_cXp(Ia - (Ia * smat * Ia) * (1 / dot_s_Ia_s));
		//Za
		GetParent()->Za += pXc_Vec(Z_plus_Ia_c + (Ia_s * (torque - dot_s_Z_plus_Ia_c)) / dot_s_Ia_s);
	}
}

////////////////////////////////////////////////////////
//E֐߉xvZElϕ
//ExvZ
void PHJoint::Integrate(double dt)
{
	//񃋁[gm[h̏ꍇ
	if(GetParent()){
		a_p = cXp_Vec(GetParent()->a);
		bool bOutOfRange = false;
		//͈͐Lȏꍇ
		/*
		if(info.fMaxPosition > info.fMinPosition){
			if(info.fPosition >= info.fMaxPosition && info.fVelocity > 0.0){
				accel = 0;
				info.fVelocity = 0;				//xO
				info.fPosition = info.fMaxPosition;
				bOutOfRange = true;
			}
			else if(info.fPosition <= info.fMinPosition && info.fVelocity < 0.0){
				accel = 0;
				info.fVelocity = 0;
				info.fPosition = info.fMinPosition;
				bOutOfRange = true;
			}
		}*/
		if(!bOutOfRange){
			//xvZ
			accel = (torque - svdot(s, Ia * a_p) - dot_s_Z_plus_Ia_c) / dot_s_Ia_s;	
			//ʒuϕ
			info.fPosition += float ( (info.fVelocity + 0.5 * accel * dt) * dt );
			//]֐߂̏ꍇ[-,]
			if(info.nType == TYPE_HINGE){
				LimitCycle(info.fPosition);
			}
			//xϕ
			info.fVelocity += float(accel * dt);
		}
		//dS̉x(qm[h̐ϕŎgp)
		a = a_p + c + accel * s;
		
		//ʒuEx̓`d
		PropagateState();		
	}
	//[gm[h̏ꍇ
	else{
		//physical
		if(solid){
			//xvZ
			a = (Ia.inv() * Za) * -1;
			//xω
			Vec3d dv_abs = R * svitem(a, 1) * dt;
			//pxω
			Vec3d dw_abs = R * svitem(a, 0) * dt;

			//ʒuXV
			p += (v_abs + 0.5 * dv_abs) * dt;
			//]ʂXV
			quat += quat.derivative(w_abs + 0.5 * dw_abs) * dt;
			quat.unitize();
			quat.to_matrix(R);

			//xXV
			v_abs += dv_abs;
			w_abs += dw_abs;
			v = quat.conjugated() * v_abs;
			w = quat.conjugated() * w_abs;

			Vec3d a_rot = R * svitem(a, 0);
			Vec3d a_trn = R * svitem(a, 1);

			/* `
			//xϕĐVx߂
			Vec3d w_new = (w_abs + a_rot * dt);
			Vec3d v_new = (v_abs + a_trn * dt);

			//xϕĈʒu߂
			p += (v_abs + (0.5 * dt) * (R * a_trn)) * dt;

			double wnorm = w.norm();
			//N[^jI]
			if(wnorm > 1.0e-10){
				quat = Quaterniond::Rot(wnorm * dt, w_abs / wnorm) * quat;
				quat.unitize();
			}
			//]sɕϊ
			quat.to_matrix(R);

			//xXV
			v_abs = v_new;
			w_abs = w_new;
			v = R.trans() * v_abs;
			w = R.trans() * w_abs;
			*/
		}
		//non-physical
		else{	//if(frame)
			a.clear();
		}
	}

	//֘AR|[lg̈ʒuAxA֐ߕψʁA֐ߑxXV
	if(solid){
#if 0	//	hase
		if (pos.norm() > 100){
			DSTR << "Strange position:" << std::endl;
			DSTR << p << std::endl;
			DSTR << R << solid->GetCenter() << std::endl;
			DSTR << GetParent()->p << std::endl;
		}
#endif
		solid->SetCenterPosition(p);
		solid->SetRotation(R);
		solid->SetVelocity(v_abs);
		solid->SetAngularVelocity(w_abs);
		solid->SetIntegrationMode(PHINT_NONE);
	}

	//̃[v̂߂ɃNA
	Ia.clear();
	Za.clear();
	
	for(array_type::const_iterator it = Children().begin(); it != Children().end(); it++)
		(*it)->Integrate(dt);
}

void PHJoint::PropagateState()
{
	//em[hƂ̑ΈʒuApvZ
	CompRelativePosition();

	//ΈʒuΈʒuvZ
	R = GetParent()->R * pRc;
	p = GetParent()->p + R * prc;

	//em[h̑xEpx{֐ߑx@@xEpx
	w = cRp * GetParent()->w;
	v = cRp * GetParent()->v + cross(w, prc);
	CompRelativeVelocity();
	v += pvc;
	w += pwc;
	v_abs = R * v;
	w_abs = R * w;
}

/////////////////////////////////////////////////////////////////////

class PHJointState: public SGBehaviorState{
public:
	SGOBJECTDEF(PHJointState);
	//񃋁[gm[h̏
	float fPosition;
	float fVelocity;
	float fInput;
	double error;		//덷ƌ덷ݐς͕ۑ
	double error_int;
	//[gm[h̏
	Vec3d p;
	Matrix3d R;
	Vec3d v_abs;
	Vec3d w_abs;
};
SGOBJECTIMP(PHJointState, SGBehaviorState);

void PHJoint::LoadState(const SGBehaviorStates& states){
	PHJointState* js = DCAST(PHJointState, states.GetNext());
	
	//[h
	if(GetParent()){
		info.fPosition = js->fPosition;
		info.fVelocity = js->fVelocity;
		info.fInput = js->fInput;
		//error_diff = js->error_diff;
		error = js->error;
		error_int = js->error_int;	
	}
	else{
		p = js->p;
		R = js->R;
		v_abs = js->v_abs;
		w_abs = js->w_abs;
	}

	Reset();

	//	qWCg[h
	for(unsigned i=0; i<Children().size(); ++i){
		Children()[i]->LoadState(states);
	}
}

void PHJoint::SaveState(SGBehaviorStates& states) const{
	UTRef<PHJointState> js = new PHJointState;
	states.push_back(js);
	if(GetParent()){
		js->fPosition = info.fPosition;
		js->fVelocity = info.fVelocity;
		js->fInput = info.fInput;
		js->error = error;
		js->error_int = error_int;
	}
	else{
		js->p = p;
		js->R = R;
		js->v_abs = v_abs;
		js->w_abs = w_abs;
	}
	for(unsigned i=0; i<Children().size(); ++i){
		Children()[i]->SaveState(states);
	}
}

//////////////////////////////////////////////////////////////////////
//PHJointCalcSlider
void PHJointCalcSlider::CompJointAxis()
{
	svitem(joint->s, 0).clear();
	svitem(joint->s, 1) = joint->GetInfo().m3fRotationChild.Ez();
}

void PHJointCalcSlider::CompRelativePosition(){
	TMatrixRow<3,3,float,float,float> work_around_for_bcb;
	joint->pRc = joint->GetInfo().m3fRotationParent * joint->GetInfo().m3fRotationChild.trans();
	joint->cRp = joint->pRc.trans();
	Vec3d cp;
	if(joint->GetParent()->solid)cp = joint->GetParent()->solid->GetCenter();
	joint->prc = (joint->cRp * (joint->GetInfo().v3fPositionParent - cp)) +
		joint->GetInfo().m3fRotationChild.Ez() * joint->GetInfo().fPosition - (joint->GetInfo().v3fPositionChild - joint->solid->GetCenter());
}

void PHJointCalcSlider::CompRelativeVelocity()
{
	joint->pvc = joint->GetInfo().m3fRotationChild.Ez() * joint->GetInfo().fVelocity;
	joint->pwc.clear();
}

void PHJointCalcSlider::CompCoriolisAccel()
{
	Vec3d wp = joint->cRp * joint->GetParent()->w;
	svitem(joint->c, 0).clear();
	svitem(joint->c, 1) = cross(wp, cross(wp, joint->prc)) + 2.0 * cross(wp, joint->GetInfo().m3fRotationChild.Ez() * joint->GetInfo().fVelocity);
}

//////////////////////////////////////////////////////////////////////
//PHJointCalcHinge
void PHJointCalcHinge::CompJointAxis()
{
	svitem(joint->s, 0) = joint->GetInfo().m3fRotationChild.Ez();
	svitem(joint->s, 1) = cross(joint->GetInfo().m3fRotationChild.Ez(), -(joint->GetInfo().v3fPositionChild - joint->solid->GetCenter()));
}

void PHJointCalcHinge::CompRelativePosition()
{
	joint->pRc = joint->GetInfo().m3fRotationParent * Matrix3d::Rot(joint->GetInfo().fPosition, 'z') * joint->GetInfo().m3fRotationChild.trans();
	joint->cRp = joint->pRc.trans();
	Vec3d cp;
	if(joint->GetParent()->solid) joint->GetParent()->solid->GetCenter();
	joint->prc = (joint->cRp * (joint->GetInfo().v3fPositionParent - cp)) - (joint->GetInfo().v3fPositionChild - joint->solid->GetCenter());
}

void PHJointCalcHinge::CompRelativeVelocity()
{
	joint->pvc = joint->GetInfo().fVelocity * svitem(joint->s, 1);
	joint->pwc = joint->GetInfo().fVelocity * svitem(joint->s, 0);
}

void PHJointCalcHinge::CompCoriolisAccel()
{
	Vec3d ud = joint->GetInfo().m3fRotationChild.Ez() * joint->GetInfo().fVelocity;
	Vec3d wp = joint->cRp * joint->GetParent()->w;
	Vec3d tmp = cross(ud, (joint->GetInfo().v3fPositionChild - joint->solid->GetCenter()));
	svitem(joint->c, 0) = cross(wp, ud);
	svitem(joint->c, 1) = cross(wp, cross(wp, joint->prc)) - 2.0 * cross(wp, tmp) - cross(ud, tmp);
}

/*			//{[WCg͉]WCgRŎ
			UTRef<AC3Joint> joint[3];
			AC3JointInfo i;

			joint[0] = new AC3RevolutiveJoint(0, BallJoint.v3PositionParent, Vec3d(), Vec3d(1.0, 0.0, 0.0));
			joint[1] = new AC3RevolutiveJoint(0, Vec3d(), Vec3d(), Vec3d(0.0, 1.0, 0.0));
			joint[2] = new AC3RevolutiveJoint(pChild, Vec3d(), BallJoint.v3PositionChild, Vec3d(0.0, 0.0, 1.0));
			joint[0]->AddChild(joint[1]);
			joint[1]->AddChild(joint[2]);
			return _Connect(pParent, joint[0]);		
*/

////////////////////////////////////////////////////////////////////

SGOBJECTIMP(PHJointEngine, SGBehaviorEngine);
void PHJointEngine::Loaded(SGScene* scene){
	root->Loaded(scene);
}
void PHJointEngine::Step(SGScene* scene)
{
	timer.Start();
	root->Dynamics(scene->GetTimeStep());
	timer.Stop();
}

SGObject* PHJointEngine::ChildObject(size_t i){
	return root;
}
size_t PHJointEngine::NChildObjects(){
	return root ? 1 : 0;
}
bool PHJointEngine::AddChildObject(SGObject* o, SGScene* s){
	if (DCAST(PHSolid, o)){
		root->solid = (PHSolid*)o;
		return true;
	}
	if (DCAST(SGFrame, o)){
		root->frame= (SGFrame*)o;
		return true;
	}
	if (DCAST(PHJoint, o)){
		((PHJoint*)o)->SetParent(root);
		return true;
	}
	return false;
}
bool PHJointEngine::DelChildObject(SGObject* o, SGScene* s){
	if (root == o){
		root = NULL;
		return true;
	}
	return false; 
}
const UTTypeInfo** PHJointEngine::ChildCandidates(){
	static const UTTypeInfo* rv[] = {
		PHJoint::GetTypeInfoStatic(), NULL
	};
	return rv;
}
void PHJointEngine::LoadState(const SGBehaviorStates& states){
	root->LoadState(states);
}
void PHJointEngine::SaveState(SGBehaviorStates& states) const{
	root->SaveState(states);
}

//////////////////////////////////////////////////////////////////////
class PHJointEngineLoader : public FIObjectLoader<PHJointEngine>{
public:
	bool LoadData(FILoadScene* ctx, PHJointEngine* engine){
		engine->root = new PHJoint;
		return true;
	}
};

class PHJointEngineSaver : public FIBaseSaver
{
public:
	UTString GetType() const{ return "PHJointEngine";}
	void Save(FISaveScene* ctx, SGObject* arg){
		PHJointEngine* je = (PHJointEngine*)arg;
		FIDocNodeBase* doc = ctx->CreateDocNode("JointEngine", je);
		//ڑΏۂۑ
		if (je->root->solid){
			doc->AddChild(ctx->CreateDocNode("REF", je->root->solid));
		}else if(je->root->frame){
			doc->AddChild(ctx->CreateDocNode("REF", je->root->frame));
		}

		ctx->docs.back()->AddChild(doc);
		ctx->docs.push_back(doc);
		for(unsigned i=0; i<je->root->Children().size(); ++i){
			ctx->SaveRecursive(je->root->Children()[i]);
		}
		ctx->docs.pop_back();
	}
};

///////////////////////////////////////////////////////////////////////

class PHJointLoader : public FIObjectLoader<PHJoint>
{
protected:
	bool LoadData(FILoadScene* ctx, PHJoint* joint){		
		//֐߂̎ނƃp[^
		PHJointInfo info;
		ctx->docs.Top()->GetWholeData(info);
		//	nEnϊ
		info.m3fRotationChild.ExZ() *= -1;
		info.m3fRotationChild.EyZ() *= -1;
		info.m3fRotationChild.EzX() *= -1;
		info.m3fRotationChild.EzY() *= -1;
		info.v3fPositionChild.Z() *= -1;
		info.m3fRotationParent.ExZ() *= -1;
		info.m3fRotationParent.EyZ() *= -1;
		info.m3fRotationParent.EzX() *= -1;
		info.m3fRotationParent.EzY() *= -1;
		info.v3fPositionParent.Z() *= -1;
		joint->info = info;
		
		switch((int)(joint->info.nType)){
		case TYPE_HINGE:	joint->jointCalc = new PHJointCalcHinge;	break;
		case TYPE_SLIDER:	joint->jointCalc = new PHJointCalcSlider;	break;
		default: ;
		}
		joint->jointCalc->joint = joint;
		return true;
	}
};

class PHJointSaver : public FIObjectSaver<PHJoint>
{
protected:
	void SaveData(class FISaveScene* ctx, FIDocNodeBase* doc, PHJoint* joint){
		PHJointInfo info = joint->info;
		//	nEnϊ
		info.m3fRotationChild.ExZ() *= -1;
		info.m3fRotationChild.EyZ() *= -1;
		info.m3fRotationChild.EzX() *= -1;
		info.m3fRotationChild.EzY() *= -1;
		info.v3fPositionChild.Z() *= -1;
		info.m3fRotationParent.ExZ() *= -1;
		info.m3fRotationParent.EyZ() *= -1;
		info.m3fRotationParent.EzX() *= -1;
		info.m3fRotationParent.EzY() *= -1;
		info.v3fPositionParent.Z() *= -1;

		doc->SetWholeData(info);
	}
};

DEF_REGISTER_BOTH(PHJoint);
DEF_REGISTER_BOTH(PHJointEngine);

}
